[![Sensu Bonsai Asset](https://img.shields.io/badge/Bonsai-Download%20Me-brightgreen.svg?colorB=89C967&logo=sensu)](https://bonsai.sensu.io/assets/sensu/sensu-bigpanda-handler)
![goreleaser](https://github.com/sensu/sensu-bigpanda-handler/workflows/goreleaser/badge.svg)
[![Go Test](https://github.com/sensu/sensu-bigpanda-handler/workflows/Go%20Test/badge.svg)](https://github.com/sensu/sensu-bigpanda-handler/actions?query=workflow%3A%22Go+Test%22)
[![goreleaser](https://github.com/sensu/sensu-bigpanda-handler/workflows/goreleaser/badge.svg)](https://github.com/sensu/sensu-bigpanda-handler/actions?query=workflow%3Agoreleaser)


# Sensu BigPanda Handler

## Table of Contents
- [Overview](#overview)
- [Usage examples](#usage-examples)
  - [Help output](#help-output)
  - [Environment variables](#environment-variables)
  - [Templates](#templates)
  - [Annotations](#annotations)
- [Configuration](#configuration)
  - [Asset registration](#asset-registration)
  - [Handler definition](#handler-definition)
  - [Check definition](#check-definition)
- [Local testing and troubleshooting](#local-testing-and-troubleshooting)
- [Installation from source and contributing](#installation-from-source-and-contributing)


## Overview

The [Sensu BigPanda Handler][0] is a [Sensu Event Handler][1] that sends alerts to
the [BigPanda service][2].


### Sensu Event to BigPand Alert Attribute translation
The common BigPanda alerts attributes are filled using the following rules: 

|Required BigPanda Alert Attributes         |Description                |
|-------------------------------------------|-------------------------------------|
| host                                      | sensu event entity name
| check                                     | sensu event check name
| cluster                                   | sensu event namespace
| timestamp                                 | sensu event timestamp
| status                                    | sensu event check state mapped into BigPanda status ("ok","warning","critical","unknown")
| primary_property                          | default to "host" settable via cmdline argument
| secondary_property                        | default to "check" settable via cmdline argument
| description                               | filled using golang template defaults to event check output, settable via cmdline argument

### Extended Sensu Event to BigPanda Alert Attribute Translation
The BigPanda alert allows for custom attributes. This handler uses this feature to extend BigPanda alerts with sensu event contextual information:

|Extended BigPanda Alert Attributes         |Sensu Event Attribute                |
|-------------------------------------------|-------------------------------------|
|sensu.event.labels.<key>                   | event labels 
|sensu.event.attributes.<key>               | event attributes
|sensu.event.entity.labels.<key>            | event entity labels 
|sensu.event.entity.attributes.<key>        | event entity attributes
|sensu.event.check.labels.<key>             | event check labels 
|sensu.event.check.attributes.<key>         | event check Attributes
|sensu.event.check.occurrences              | event check occurrences count
|sensu.event.check.state                    | event check state ("passing", "failing", "flapping"")
|sensu.event.check.status                   | event check status (0-254)
|sensu.event.check.subscriptions            | comma separated list of event check subscriptions  
|sensu.event.entity.system.arch             | event entity system arch
|sensu.event.entity.system.cloud_provider   | event entity system cloud_provider
|sensu.event.entity.system.hostname         | event entity system hostname
|sensu.event.entity.system.os               | event entity system os
|sensu.event.entity.system.platform         | event entity system platform
|sensu.event.entity.system.platform_family  | event entity system platform_family
|sensu.event.entity.system.platform_version | event entity system platform version
|sensu.event.entity.system.vm_role          | event entity system vm_role
|sensu.event.entity.system.vm_system        | event entity system vm_version
|sensu.event.id                             | event id
 
## Usage examples

### Help output

```
Sensu Handler to send alerts to BigPanda

Usage:
  sensu-bigpanda-handler [flags]
  sensu-bigpanda-handler [command]

Available Commands:
  help        Help about any command
  version     Print the version number of this plugin

Flags:
  -t, --token string                         BigPanda API Authentication Token - Required. Note for security using BIGPANDA_TOKEN environment variable is preferred
  -a, --appkey string                        BigPanda API App Key - Required . Note for security using BIGPANDA_APPKEY environment variable is preferred
  -c, --custom-tags strings                  Comma separated list of tags to add using 'tag_name: tag_value'  format
  -d, --description-template string          The BigPanda alert description template, in Golang text/template format (default "{{ .Check.Output }}")
  -p, --primary-property string              The BigPanda alert primary property (default "host")
  -s, --secondary-property string            The BigPanda alert secondary property (default "check")
  -S, --status string                        The BigPanda alert status, overrides check status
  -u, --url string                           The BigPanda alert API URL (default "https://api.bigpanda.io/data/v2/alerts")
  -v, --verbose                              Enable verbose output
  -n, --dry-run                              Dry run, do not contact BigPanda. For testing only, implies --verbose
  -h, --help                                 help for sensu-bigpanda-handler

Use "sensu-bigpanda-handler [command] --help" for more information about a command.
```
***Note:*** You must generate both an http authentication token and app key via the Big Panda service by following the steps to enable a new webhook integration for the Alert API. 

### Environment variables

|Argument                       |Environment Variable                 |
|-------------------------------|-------------------------------------|
|--token                        |BIGPANDA_TOKEN                       |
|--appkey                       |BIGPANDA_APPKEY                      |
|--url                          |BIGPANDA_URL                         |
|--status                       |BIGPANDA_STATUS                      |
|--description-template         |BIGPANDA_DESCRIPTION_TEMPLATE        |
|--primary-property             |BIGPANDA_PRIMARY_PROPERTY            |
|--secondary-property           |BIGPANDA_SECONDARY_PROPERTY          |
|--custom-tags                  |BIGPANDA_CUSTOM_TAGS                 |

**Security Note:** Care should be taken to not expose the token/appkey authentication
credentials for this handler by specifying them on the command line or by directly setting the environment
variable in the handler definition.  It is suggested to make use of [secrets management][3] to surface these as
an environment variables.  The handler definition referenced here uses secrets for the authentication credentials.
Below is an example secrets definition that make use of the built-in [env secrets provider][4].

```yml
---
type: Secret
api_version: secrets/v1
metadata:
  name: bigpanda_token
spec:
  provider: env
  id: BIGPANDA_TOKEN
---
type: Secret
api_version: secrets/v1
metadata:
  name: bigpanda_appkey
spec:
  provider: env
  id: BIGPANDA_APPKEY
```

### Templates

This handler provides options for using GoLang templates to optionally populate several BigPanda alert attributes.
For example, by default the alert description is populated using a template to extract the Sensu checkout output. 
Similarly the primary property, secondary property and status attributes can also use templates if required.
More information on template syntax and format can be found in [the documentation][5]

### Annotations

All arguments for this handler are tunable on a per entity or check basis based
on annotations. The annotations keyspace for this handler is
`sensu.io/plugins/bigpanda/config`.

**NOTE**: Due to [check token substituion][6], supplying a template value such
as for `description-template` as a check annotation requires that you place the
desired template as a [golang string literal][7] (enlcosed in backticks)
within another template definition.  This does not apply to entity annotations.

#### Examples

To customize the custom tags for a specific entity, you could use the following
sensu-agent configuration snippet:

```yml
# /etc/sensu/agent.yml example
annotations:
  sensu.io/plugins/bigpanda/config/custom-tags: 'enviroment: testing, application: infinidash'
```

## Configuration

### Asset registration

[Sensu Assets][8] are the best way to make use of this plugin. If you're not using an asset, please
consider doing so! If you're using sensuctl 5.13 with Sensu Backend 5.13 or later, you can use the
following command to add the asset:

```
sensuctl asset add sensu/sensu-bigpanda-handler
```

If you're using an earlier version of sensuctl, you can find the asset on the [Bonsai Asset Index][https://bonsai.sensu.io/assets/sensu/sensu-bigpanda-handler].

### Handler definition

```yml
---
api_version: core/v2
type: Handler
metadata:
  namespace: default
  name: bigpanda
spec:
  type: pipe
  command: sensu-bigpanda-handler 
  filters:
  - is_incident
  runtime_assets:
  - sensu/sensu-bigpanda-handler
  secrets:
  - name: BIGPANDA_TOKEN
    secret: bigpanda_token
  - name: BIGPANDA_APPKEY
    secret: bigpanda_appkey
  timeout: 10
```
**Security Note**: The BigPanda authentication credentials token/appkey
should always be treated as a security sensitive configuration options and in this example,
they are loaded into the handler configuration as an environment variable using a [secret][4].
Command arguments are commonly readable from the process table by other unprivaledged
users on a system (ex: ps and top commands), so it's a better practise to read
in sensitive information via environment variables or configuration files on
disk. The cmdline argument flags for credentials are provided as overrides for testing purposes.

### Check definition

```
api_version: core/v2
type: CheckConfig
metadata:
  namespace: default
  name: dummy-app-healthz
spec:
  command: check-http -u http://localhost:8080/healthz
  subscriptions:
  - dummy
  publish: true
  interval: 10
  handlers:
  - bigpanda
```


#### Proxy Support

This handler supports the use of the environment variables HTTP_PROXY,
HTTPS_PROXY, and NO_PROXY (or the lowercase versions thereof). HTTPS_PROXY takes
precedence over HTTP_PROXY for https requests.  The environment values may be
either a complete URL or a "host[:port]", in which case the "http" scheme is assumed.

## Local testing and troubleshooting

Before deploying as a handler in your Sensu environment, you can optionally test for correct functionality using the handler executable locally in your development environemnt.
First download and unpack the appropriate asset tarball for the development host and change into the 'bin/' of the unpacked tarball.

You can test by sending an event into stdin of the executable using sensuctl:
```
sensuctl event info <ENTITY_NAME> <CHECK_NAME> --format json | ./sensu-bigpanda-handler --token <TOKEN> --appkey <APPKEY> --dry-run
```

The `--dry-run` argument will prevent the executable from contacting the bigpanda api and will instead report back intended api communication.  Remove the `--dry-run` option to send the msg to the BigPand api.

 
## Installation from source

Download the latest version of the sensu-bigpanda-handler from [releases][9],
or create an executable from this source.

### Compiling

From the local path of the sensu-bigpanda-handler repository:
```
go build
```

## Additional notes

## Contributing

To contribute to this plugin, see [CONTRIBUTING](https://github.com/sensu/sensu-go/blob/master/CONTRIBUTING.md)

[0]: https://github.com/sensu/sensu-bigpanda-handler
[1]: https://docs.sensu.io/sensu-go/latest/reference/handlers/#how-do-sensu-handlers-work
[2]: https://www.bigpanda.io/
[3]: https://docs.sensu.io/sensu-go/latest/guides/secrets-management/
[4]: https://docs.sensu.io/sensu-go/latest/guides/secrets-management/#use-env-for-secrets-management
[5]: https://docs.sensu.io/sensu-go/latest/observability-pipeline/observe-process/handler-templates/
[6]: https://docs.sensu.io/sensu-go/latest/observability-pipeline/observe-schedule/checks/#check-token-substitution
[7]: https://golang.org/ref/spec#String_literals
[8]: https://docs.sensu.io/sensu-go/latest/reference/assets/
[9]: https://github.com/sensu/sensu-bigpanda-handler/releases

